@@ -60,11 +60,12 @@ private slots:
     void testMapViewBoxToTarget();
     void testRenderElement();
     void testRenderElementToBounds();
+    void testRenderDocumentWithSizeToBounds();
     void constructorQXmlStreamReader() const;
     void loadQXmlStreamReader() const;
     void nestedQXmlStreamReader() const;
     void stylePropagation() const;
-    void matrixForElement() const;
+    void transformForElement() const;
     void boundsOnElement() const;
     void gradientStops() const;
     void gradientRefs();
@@ -73,6 +74,7 @@ private slots:
     void fillRule();
     void opacity();
     void paths();
+    void paths2();
     void displayMode();
     void strokeInherit();
     void testFillInheritance();
@@ -81,6 +83,10 @@ private slots:
     void smallFont();
     void styleSheet();
     void duplicateStyleId();
+    void oss_fuzz_23731();
+    void oss_fuzz_24131();
+    void oss_fuzz_24738();
+    void imageRendering();
 
 #ifndef QT_NO_COMPRESS
     void testGzLoading();
@@ -269,11 +275,13 @@ void tst_QSvgRenderer::testMapViewBoxToTarget()
         QCOMPARE(picture.boundingRect(), QRect(125, 125, 250, 250));
     }
 
+    // Requires keep-aspectratio feature
     { // Viewport and viewBox specified -> scale 500x500 square to 1000x750 while preserving aspect ratio gives 750x750
         data = "<svg width=\"1000\" height=\"750\" viewBox=\"-250 -250 500 500\"><g><rect x=\"0\" y=\"0\" width=\"500\" height=\"500\" /></g></svg>";
         QPicture picture;
         QPainter painter(&picture);
         QSvgRenderer rend(data);
+        rend.setAspectRatioMode(Qt::KeepAspectRatio);
         rend.render(&painter);
         painter.end();
         QCOMPARE(picture.boundingRect(), QRect(500, 375, 750, 750));
@@ -372,6 +380,36 @@ void tst_QSvgRenderer::testRenderElementToBounds()
     QCOMPARE(reference, rendering);
 }
 
+void tst_QSvgRenderer::testRenderDocumentWithSizeToBounds()
+{
+    // QTBUG-80888
+    QImage reference(400, 200, QImage::Format_ARGB32);
+    {
+        reference.fill(Qt::transparent);
+        QPainter p(&reference);
+        p.fillRect(100, 100, 100,  50, Qt::blue);
+        p.fillRect(200,  50, 100,  50, Qt::blue);
+    }
+
+    QImage rendering(400, 200, QImage::Format_ARGB32);
+    {
+        const char *const src = R"src(
+        <svg width="20" height="80">
+            <g transform="translate(-100,-100)">
+                <path d="m 110,180 v -80 h 10 v 40 h -20 v 40 z" fill="blue" />
+            </g>
+        </svg>
+        )src";
+        const QByteArray data(src);
+        QSvgRenderer rend(data);
+        rendering.fill(Qt::transparent);
+        QPainter p(&rendering);
+        rend.render(&p, QRectF(100, 50, 200, 100));
+    }
+
+    QCOMPARE(reference, rendering);
+}
+
 void tst_QSvgRenderer::constructorQXmlStreamReader() const
 {
     const QByteArray data(src);
@@ -486,14 +524,17 @@ static qreal transformNorm(const QTransform &m)
         + m.m33() * m.m33());
 }
 
-static bool diffIsSmallEnough(double diff, double norm)
+template<typename T>
+static inline bool diffIsSmallEnough(T diff, T norm)
 {
-    return diff <= 1e-12 * norm;
-}
-
-static inline bool diffIsSmallEnough(float diff, float norm)
-{
-    return diff <= 1e-5 * norm;
+    static_assert(std::is_same_v<T, double> || std::is_same_v<T, float>);
+    T sigma = []{
+        if constexpr (std::is_same_v<T, double>)
+            return 1e-12;
+        else
+            return 1e-5;
+    }();
+    return diff <= sigma * norm;
 }
 
 static void compareTransforms(const QTransform &m1, const QTransform &m2)
@@ -512,7 +553,7 @@ static void compareTransforms(const QTransform &m1, const QTransform &m2)
     QVERIFY(diffIsSmallEnough(diffNorm, qMin(norm1, norm2)));
 }
 
-void tst_QSvgRenderer::matrixForElement() const
+void tst_QSvgRenderer::transformForElement() const
 {
     QByteArray data("<svg>"
                       "<g id='ichi' transform='translate(-3,1)'>"
@@ -530,15 +571,15 @@ void tst_QSvgRenderer::matrixForElement() const
     QPainter painter(&image);
     QSvgRenderer renderer(data);
 
-    compareTransforms(painter.worldTransform(), QTransform(renderer.matrixForElement(QLatin1String("ichi"))));
+    compareTransforms(painter.worldTransform(), renderer.transformForElement(QLatin1String("ichi")));
     painter.translate(-3, 1);
-    compareTransforms(painter.worldTransform(), QTransform(renderer.matrixForElement(QLatin1String("ni"))));
+    compareTransforms(painter.worldTransform(), renderer.transformForElement(QLatin1String("ni")));
     painter.rotate(45);
-    compareTransforms(painter.worldTransform(), QTransform(renderer.matrixForElement(QLatin1String("san"))));
+    compareTransforms(painter.worldTransform(), renderer.transformForElement(QLatin1String("san")));
     painter.scale(4, 2);
-    compareTransforms(painter.worldTransform(), QTransform(renderer.matrixForElement(QLatin1String("yon"))));
+    compareTransforms(painter.worldTransform(), renderer.transformForElement(QLatin1String("yon")));
     painter.setWorldTransform(QTransform(1, 2, 3, 4, 5, 6), true);
-    compareTransforms(painter.worldTransform(), QTransform(renderer.matrixForElement(QLatin1String("firkant"))));
+    compareTransforms(painter.worldTransform(), renderer.transformForElement(QLatin1String("firkant")));
 }
 
 void tst_QSvgRenderer::boundsOnElement() const
@@ -803,10 +844,9 @@ void tst_QSvgRenderer::testGzHelper_data()
             "cbcfe70200a865327e040000001f8b08001c2a934800034b4a2ce20200e9b3a20404000000"))
         << QByteArray("foo\nbar\n");
 
-    // We should still get data of the first member if subsequent members are corrupt
     QTest::newRow("corruptedSecondMember") << QByteArray::fromHex(QByteArray("1f8b08001c2a934800034b"
             "cbcfe70200a865327e040000001f8c08001c2a934800034b4a2ce20200e9b3a20404000000"))
-        << QByteArray("foo\n");
+        << QByteArray();
 
 }
 
@@ -1012,6 +1052,19 @@ void tst_QSvgRenderer::paths()
     }
 }
 
+void tst_QSvgRenderer::paths2()
+{
+    const char *svg =
+        "<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"16\" height=\"16\">"
+            "<path d=\"M 3 8 A 5 5 0 1013 8\" id=\"path1\"/>"
+        "</svg>";
+
+    QByteArray data(svg);
+    QSvgRenderer renderer(data);
+    QVERIFY(renderer.isValid());
+    QCOMPARE(renderer.boundsOnElement(QLatin1String("path1")).toRect(), QRect(3, 8, 10, 5));
+}
+
 void tst_QSvgRenderer::displayMode()
 {
     static const char *svgs[] = {
@@ -1572,5 +1625,77 @@ void tst_QSvgRenderer::duplicateStyleId()
     renderer.render(&painter);
 }
 
+void tst_QSvgRenderer::oss_fuzz_23731()
+{
+    // when configured with "-sanitize undefined", this resulted in:
+    // "runtime error: division by zero"
+    QSvgRenderer().load(QByteArray("<svg><path d=\"A4------\">"));
+}
+
+void tst_QSvgRenderer::oss_fuzz_24131()
+{
+    // when configured with "-sanitize undefined", this resulted in:
+    // "runtime error: -nan is outside the range of representable values of type 'int'"
+    // runtime error: signed integer overflow: -2147483648 + -2147483648 cannot be represented in type 'int'
+    QImage image(377, 233, QImage::Format_RGB32);
+    QPainter painter(&image);
+    QSvgRenderer renderer(QByteArray("<svg><path d=\"M- 4 44044404444E-334-\"/></svg>"));
+    renderer.render(&painter);
+}
+
+void tst_QSvgRenderer::oss_fuzz_24738()
+{
+    // when configured with "-sanitize undefined", this resulted in:
+    // "runtime error: division by zero"
+    QSvgRenderer().load(QByteArray("<svg><path d=\"a 2 1e-212.....\">"));
+}
+
+QByteArray image_data_url(QImage &image) {
+    QByteArray data;
+    QBuffer buffer(&data);
+    buffer.open(QBuffer::ReadWrite);
+    image.save(&buffer, "PNG");
+    buffer.close();
+    QByteArray url("data:image/png;base64,");
+    url.append(data.toBase64());
+    return url;
+}
+
+void tst_QSvgRenderer::imageRendering() {
+    QImage img(2, 2, QImage::Format_ARGB32_Premultiplied);
+    img.fill(Qt::green);
+    img.setPixel(0, 0, qRgb(255, 0, 0));
+    img.setPixel(1, 1, qRgb(255, 0, 0));
+    QByteArray imgurl(image_data_url(img));
+    QString svgtemplate(
+        "<svg><g transform='scale(2, 2)'>"
+            "<image image-rendering='%1' xlink:href='%2' width='2' height='2' />"
+        "</g></svg>"
+    );
+    const char *cases[] = {"optimizeQuality", "optimizeSpeed"};
+    for (auto ir: cases) {
+        QString svg = svgtemplate.arg(QLatin1String(ir)).arg(QLatin1String(imgurl));
+        QImage img1(4, 4, QImage::Format_ARGB32);
+        QPainter p1;
+        p1.begin(&img1);
+        QSvgRenderer renderer(svg.toLatin1());
+        Q_ASSERT(renderer.isValid());
+        renderer.render(&p1);
+        p1.end();
+
+        QImage img2(4, 4, QImage::Format_ARGB32);
+        QPainter p2(&img2);
+        p2.scale(2, 2);
+        if (QLatin1String(ir) == QLatin1String("optimizeSpeed"))
+            p2.setRenderHint(QPainter::SmoothPixmapTransform, false);
+        else if (QLatin1String(ir) == QLatin1String("optimizeQuality"))
+            p2.setRenderHint(QPainter::SmoothPixmapTransform, true);
+        p2.drawImage(0, 0, img);
+        p2.end();
+        QCOMPARE(img1, img2);
+    }
+}
+
+
 QTEST_MAIN(tst_QSvgRenderer)
 #include "tst_qsvgrenderer.moc"
